
import React from 'react';
import ReactDOM from 'react-dom';
import Immutable from 'immutable';
import process from '../build/processor.js';


const occupiedStates = ['', 'formerly', 'occupied'];
const Cell = React.createClass({
    render() {
        let classname = this.props.occupied ? "cell " + occupiedStates[this.props.occupied] : "cell";
        return <div className={classname} style={{top:this.props.y, left:this.props.x, width:this.props.size, height:this.props.size}}></div>;
    },
});


const Board = React.createClass({
    getInitialState () {
        return {
            world: this.props.world,
            settings: this.props.settings.board,
            speed: this.props.settings.speed,
            playing: false,
        };
    },
    preAdvance() {
        requestAnimationFrame(this.advance);
    },
    advance() {
        if( this.state.playing ) {
            this.setState({world: process(this.state.world)});
        }
        setTimeout(this.preAdvance, 1000/this.state.speed);
    },
    componentDidMount() {
        this.advance();
    },
    togglePlay() {
        this.setState({playing: !this.state.playing});
    },
    render() {
        let ot = this.state.settings.offsetTop || 0,
            ol = this.state.settings.offsetLeft || 0,
            size = this.state.settings.cellSize || 10;
        let cells = this.state.world.flatMap(function (row, y) {
            return row.map(function eachCell(value, x) {
                let xc = ((x * size) + ol) + 'px',
                    yc = ((y * size) + ot) + 'px';
                return <Cell key={'cell'+x+'-'+y} x={xc} y={yc} size={size} occupied={value} />;
            });
        }).toJS();

        return <div id="panel">
            <div id="board">
                {cells}
            </div>
            <div id="controls">
                <button onClick={this.togglePlay}>{this.state.playing ? "pause" : "play"}</button>
            </div>
        </div>;
    },
});




let settings = {
    board: {
        offsetTop:40,
        offsetLeft:10,
        width:50,
        height:20,
        cellSize:10,
    },
    speed:5,
};
function generateWorld(w = 10, h = 10) {
    let grid = [];
    for( let i = 0; i < h; i++ ) {
        let row = [];
        for( let j = 0; j < w; j++ ) {
            row.push(0);
        }
        grid.push(row);
    }
    return Immutable.fromJS(grid);
}
function populateRandomly(state) {
    return state.map(function rows(row) {
        return row.map(function cols(cell) {
            let occupied = Math.random() < 0.2,
                value = occupied ? 2 : 0;
            return value;
        });
    });
}
let world = populateRandomly(generateWorld(100, 20));

ReactDOM.render(
    <Board settings={settings} world={world} />,
    document.getElementById('board')
);



